function Create(self)
	self.SearchTimer = Timer()
	self.KeyTimer = Timer()
	self.timing = 1000
	self.rounds = 4
	self.maxRange = 2200
	self.minRange = 400
	self.grav = SceneMan.GlobalAcc.Y
	self.AimDots = {}
	self.Ctrl = self:GetController()
	self.Targets = {}
	self.aimAngle = self.RotAngle + 1.57
	self.animate = 0
	
	-- Change minimum range by modifying sharpness in the code that spawns the mortar
	if self.Sharpness > self.minRange then
		self.minRange = math.min(self.Sharpness, self.maxRange/2)
	end
	
	self.Sharpness = 1
end

function Destroy(self)
	for i = 1, #self.AimDots do
		if MovableMan:ValidMO(self.AimDots[i]) then
			self.AimDots[i].ToDelete = true
		end
	end
	
	if MovableMan:ValidMO(self.Barrel) then
		self.Barrel.ToDelete = true
	end
end

function Update(self)	
	if self:IsPlayerControlled() then
		if self.KeyTimer:IsPastRealMS(150) then
			self.KeyTimer:Reset()
			
			if self.minRange < self.maxRange/2 and self.Ctrl:IsState(Controller.BODY_JUMP) then
				self.minRange = self.minRange + 50
			elseif self.minRange > 499 and self.Ctrl:IsState(Controller.BODY_CROUCH) then
				self.minRange = self.minRange - 50
			elseif self.Ctrl:IsState(Controller.MOVE_LEFT) then
				self.ToDelete = true	-- debug
			end
		end
		
		-- Update the aim
		local AimStart = Vector(self.Pos.X + self.minRange, self.Pos.Y + 10)
		for i = 1, 12 do
			if not MovableMan:ValidMO(self.AimDots[i]) or self.AimDots[i].PresetName ~= "Aim" then
				self.AimDots[i] = CreateMOSParticle("Aim", "Air Support.rte")
				MovableMan:AddParticle(self.AimDots[i])
			end
			
			self.AimDots[i].Age = 1
			self.AimDots[i].Pos = AimStart
			AimStart.Y = AimStart.Y - 10
			
			-- Wrap
			if SceneMan.SceneWrapsX and self.AimDots[i].Pos.X > SceneMan.SceneWidth then
				self.AimDots[i].Pos.X = self.AimDots[i].Pos.X - SceneMan.SceneWidth
			end
			
			if i == 6 then	-- Left side
				AimStart = Vector(self.Pos.X - self.minRange, self.Pos.Y + 10)
			end
		end
	else
		for i = 1, #self.AimDots do
			if MovableMan:ValidMO(self.AimDots[i]) then
				self.AimDots[i].ToDelete = true
				self.AimDots[i] = nil
			end
		end
	end
		
	if self.SearchTimer:IsPastSimMS(self.timing) then
		self.SearchTimer:Reset()
		self.timing = RangeRand(800, 900)
		
		local x, y, vel, v2, root, pos, neg, ang, dist, Range 
		for Act in MovableMan.Actors do
			if Act.Team ~= self.Team and Act.ID ~= self.ID then
				Range = SceneMan:ShortestDistance(self.Pos, Act.Pos+Act.Vel*(FrameMan.PPM*3), false)	-- ToF is ~4s
				dist = math.abs(Range.X)
				if dist < self.maxRange and dist > self.minRange and Act.Vel.Magnitude < 10 and
					(Act.ClassName == "AHuman" or Act.ClassName == "ACrab" or Act.ClassName == "Actor")
				then					
					if not SceneMan:CastStrengthRay(Act.Pos, Vector(0, -200), 10, Vector(), 5, -1, SceneMan.SceneWrapsX) then	-- check if the target is in cover
						x = Range.X/FrameMan.PPM
						y = Range.Y/FrameMan.PPM
						if Range.AbsRadAngle < 1.57 then	-- target is to the right
							if dist < self.maxRange/2 then
								vel = 45
							else
								vel = 55
							end
							
							v2 = vel*vel
							root = math.sqrt(v2*v2-self.grav*(self.grav*(x*x)+2*y*v2))
							pos = math.atan((v2+root)/(self.grav*x))
							neg = math.atan((v2-root)/(self.grav*x))
						else
							if dist < self.maxRange/2 then
								vel = 45
								x = x*(0.00035*dist+0.8)	-- compensate
							else
								vel = 55
								x = x*(0.00030*dist+0.7)	-- compensate
							end
							
							v2 = vel*vel
							root = math.sqrt(v2*v2-self.grav*(self.grav*-(x*x)+2*-y*v2))
							pos = 3.1415 - math.atan((v2+root)/(self.grav*-x))
							neg = 3.1415 - math.atan((v2-root)/(self.grav*-x))
						end

						ang = nil
						if pos > -1.58 then	-- if the target is out of range the result will be the negative infinity
							ang = pos
						elseif neg > -1.58 then
							ang = neg
						end

						if ang then
							table.insert(self.Targets, {angle=ang, vel=vel})
							self.timing = 10000	-- Search again after two rounds have been fired
							if #self.Targets > 1 then	-- Try to find two targets
								break
							end
						end
					end
				end
			end
		end
	end
	
	-- Validate barrel
	self.aimAngle = math.min(math.max(self.aimAngle, self.RotAngle + 0.785), self.RotAngle + 2.356)
	if not MovableMan:ValidMO(self.Barrel) or self.Barrel.PresetName ~= "Mortar Tubes" then
		self.Barrel = CreateMOSRotating("Mortar Tubes", "Air Support.rte")
		MovableMan:AddParticle(self.Barrel)
	end
	
	self.Barrel.Age = 1
	self.Barrel.Pos = self.Pos + self:RotateOffset(Vector(0,-5))
	self.Barrel.RotAngle = self.aimAngle
	
	if self.animate == 0 then
		self.animate = 3
		if self.Barrel.Frame == 0 then
			if #self.Targets > 0 then			
				if self.Targets[1].angle > self.aimAngle then
					self.aimAngle = math.min(self.Targets[1].angle, self.aimAngle + 0.02)
				elseif self.Targets[1].angle < self.aimAngle then
					self.aimAngle = math.max(self.Targets[1].angle, self.aimAngle - 0.02)
				end
				
				if self.Targets[1].angle > self.Barrel.RotAngle - 0.005 and self.Targets[1].angle < self.Barrel.RotAngle + 0.005 then
					local Bomb = CreateAEmitter("Mortar Bomb", "Air Support.rte")
					if Bomb then
						Bomb.RotAngle = self.Barrel.RotAngle						
						Bomb.Vel = Vector(self.Targets[1].vel,0):RadRotate(self.Barrel.RotAngle+RangeRand(-0.04, 0.04))
						
						if self.rounds % 2 == 0 then
							Bomb.Pos = self.Barrel.Pos + self.Barrel:RotateOffset(Vector(10, 3))
							self.Barrel.Frame = 1
							
							if #self.Targets > 1 then
								table.remove(self.Targets, 1)
							end
						else							
							Bomb.Pos = self.Barrel.Pos + self.Barrel:RotateOffset(Vector(10, -3))
							self.Barrel.Frame = 4
							table.remove(self.Targets, 1)
						end
						
						MovableMan:AddParticle(Bomb)
						
						self.animate = 8
						self.rounds = self.rounds - 1
						if self.rounds == 2 then
							self.timing = 1500	-- Reload from magazine
							self.SearchTimer:Reset()
						elseif self.rounds < 1 then
							self.rounds = 4
							self.timing = 7000	-- Reload magazine
							self.SearchTimer:Reset()
						end
					end
				end
			end
		else
			self.Barrel:SetNextFrame()
			if self.Barrel.Frame == 4 then
				self.Barrel.Frame = 0
			end
		end
	else
		self.animate = self.animate - 1
	end
end