function Create(self)
	function GoInert(self)
		self.HitsMOs = false
		self.GetsHitByMOs = false
		self.ToDelete = true
		
		local Emitter = CreateAEmitter("Inert SAM", "Air Support.rte")
		Emitter.Pos = self.Pos
		Emitter.Vel = self.Vel
		Emitter.RotAngle = self.RotAngle
		MovableMan:AddParticle(Emitter)
	end
	
	self.LifeTimer = Timer()
	self.UpdateTimer = Timer()
	self.Grav = SceneMan.GlobalAcc * (TimerMan.DeltaTimeSecs * 3 * 0.8)	-- Calculate every third frame, GlobalAccScalar is 0.8
	self.thrust = 0.2
	self.timing = math.ceil(4 * TimerMan.DeltaTimeSecs * 1000 + 1)
	self.proximity = nil
	self.armDist = nil
	self.factor = FrameMan.PPM * TimerMan.DeltaTimeSecs * 3	-- Calculate every third frame
	
	if self.Sharpness < 255 then
		self.Target = MovableMan:GetMOFromID(self.Sharpness)
		if self.Target then
			self.armDist = self.Target.Radius + 45
			self.TargetOffset = Vector(RangeRand(0,150),0):RadRotate(RangeRand(-3.14,3.14))
		end
	else
		self.armDist = 50
	end
	
	self.Sharpness = 1
end

function Update(self)
	self.AngularVel = self.AngularVel * 0.9
	if self.LifeTimer:IsPastSimMS(3500) then	-- Create an inert copy
		GoInert(self)
		return
	elseif self.LifeTimer:IsPastSimMS(3000) then	-- Out of fuel
		self:EnableEmission(false)
	else
		local RelVel = self:UnRotateOffset(self.Vel)
		RelVel.X = RelVel.X + self.thrust	-- Acceleration
		if RelVel.X > 10 then
			RelVel.X = RelVel.X * 0.995		-- 0.5% air resistance for forward movement
		end
		
		-- Air resistance for movement Perpendicular to RotAngle
		if RelVel.Y > 6 then
			RelVel.Y = RelVel.Y * 0.93		-- 7%
		else
			RelVel.Y = RelVel.Y * 0.995		-- 0.5%
		end
		
		self.Vel = self:RotateOffset(RelVel)
	end
	
	if self.Target then
		if MovableMan:ValidMO(self.Target) then
			--Calcuate target distance
			local TargetWrapPos = SceneMan:ShortestDistance(self.Pos, self.Target.Pos + self.TargetOffset, false)
			local range = TargetWrapPos.Magnitude	-- distance to target
			
			if range < self.armDist then
				self.proximity = range		-- the missile is close, arm the proximity fuse
			elseif self.proximity and self.proximity < range then
				self:GibThis()
				return
			end
			
			if self.UpdateTimer:IsPastSimMS(self.timing) then
				self.UpdateTimer:Reset()
				self.thrust = 0.8
				
				-- Predict target pos
				local AimPoint
				local speed = self.Vel.Magnitude * math.cos(TargetWrapPos.AbsRadAngle - self.Vel.AbsRadAngle)    -- Movement in direction of the target
				local t2t = math.min((range / speed) * 0.8, 20)	-- estimated time to target
				if t2t > 3 then
					if self.PrevTrgVel then
						AimPoint = self.Target.Pos + self.TargetOffset + (self.Target.Vel + (self.Target.Vel - self.PrevTrgVel)/4) * t2t
						self.PrevTrgVel = self.Target.Vel
					else
						AimPoint = self.Target.Pos + self.Target.Vel * t2t
						self.PrevTrgVel = self.Target.Vel
					end
					
					AimPoint.Y = math.max(math.min(AimPoint.Y, SceneMan.SceneHeight), -50)	-- Todo: What about maps that Y-wrap?
				else
					AimPoint = self.Target.Pos
				end
				
				--[[ Debug
				self.Dot = CreateMOSParticle("Eye", "Debug.rte")
				self.Dot.Pos = AimPoint
				self.Dot.Vel = self.Target.Vel
				MovableMan:AddParticle(self.Dot)]]
				
				-- Simulate every third frame of the missiles flight
				local Vel = Vector(self.Vel.X, self.Vel.Y)
				local Pos = Vector(self.Pos.X, self.Pos.Y)
				local minDist = 10000
				local dist
				local Closest
				local closeRotAng = 0
				local rot = self.RotAngle
				for i = 1, 15 do
					local RelVel = self:UnRotateOffset(Vel + self.Grav)
					RelVel.X = RelVel.X + self.thrust * 3	-- Acceleration*3
					if RelVel.X > 10 then
						RelVel.X = RelVel.X * 0.985			-- Air resistance^3 for forward movement
					end
					
					-- Air resistance^3 for movement Perpendicular to RotAngle
					if RelVel.Y > 6 then
						RelVel.Y = RelVel.Y * 0.804
					else
						RelVel.Y = RelVel.Y * 0.985
					end
					
					Vel = self:RotateOffset(RelVel)
					Pos = Pos + Vel * self.factor					
					
					dist = SceneMan:ShortestDistance(AimPoint, Pos, false).Magnitude
					if dist < minDist then
						minDist = dist
						Closest = Pos
						closeRotAng = Vel.AbsRadAngle
					else
						break
					end
					
					--[[ Debug
					if not self.AimDots then
						self.AimDots = {}
					end
					
					if not MovableMan:ValidMO(self.AimDots[i]) or self.AimDots[i].PresetName == "Entity" then
						self.AimDots[i] = CreateMOSParticle("Aim", "Air Support.rte")
						MovableMan:AddParticle(self.AimDots[i])
					end
					
					self.AimDots[i].Pos = Pos]]
				end
				
				-- Turn in the direction of the target
				local angDiff = SceneMan:ShortestDistance(Closest, AimPoint, false).AbsRadAngle
				if closeRotAng > angDiff + 3.1415 then
					angDiff = angDiff - (closeRotAng-6.2832)
				elseif angDiff > closeRotAng + 3.14 then
					angDiff = angDiff - 6.2832 - closeRotAng
				else
					angDiff = angDiff - closeRotAng
				end
				
				if angDiff < -0.05 then
					self.AngularVel = self.AngularVel / 2 - 1.37
					self.thrust = 0.7
				elseif angDiff > 0.05 then
					self.AngularVel = self.AngularVel / 2 + 1.37
					self.thrust = 0.7
				else
					self.thrust = 1.0
				end
			end
		elseif self.Age > 500 then
			self.Target = nil
			GoInert(self)
		end
	end
end