local tmp = package.path
package.path = package.path .. ";Air Support.rte/?.lua"
require("common")
package.path = tmp

function Create(self)
	UAV.Initialize(self)
	
	-- specific for this craft
	self.minThrot = self.minThrot * 1.1
	self.maxThrot = self.maxThrot * 1.1
	self.throttle = self.minThrot
	self.AimDistance = self.AimDistance * 1.1
	self.payload = 8
	self.payloadRof = 400	-- time in ms between drops
	self.ammo = 200
	self.RadarTimer = Timer()
	self.TargetTimer = Timer()
	self.EnemyCraft = {}
	self.FriendlyCraft = {}
	self.drawEnemy = true
	self.radarCone = 0.7	-- radar coverage is a cone of about 80 degrees
	self.RadarTarget = nil
	
	if self.payload > 2 then
		self.Frame = 0
	elseif self.payload > 0 then
		self.Frame = 1
	end
	
	-- Find radar targets
	for Act in MovableMan.Actors do
		if (Act.ClassName == "ACRocket" or Act.ClassName == "ACDropShip" or Act.ClassName == "ACraft") and not(Act:IsStatus(Actor.DYING) or Act:IsDead()) then
			if Act.Team ~= self.Team then
				table.insert(self.EnemyCraft, Act)
			elseif Act.ID ~= self.ID then
				table.insert(self.FriendlyCraft, Act)
			end
		end
	end
	
	self.RadarTimer:Reset()
end

function Destroy(self)
	UAV.Destroy(self)
end

function Update(self)
	UAV.Move(self)
	
	if self:IsPlayerControlled() then
		UAV.PlayerControlled(self)
		UAV.Map(self)
		
		-- specific for this craft
		if self.DropTimer:IsPastSimMS(self.payloadRof) and self.Ctrl:IsState(self.secondary) then
			if self.payload > 0 then
				self.DropTimer:Reset()
				
				-- Fire Missile
				local Missile = CreateAEmitter("AA Missile")
				Missile.Pos = self.Pos + self:RotateOffset(Vector(5, math.abs(self.SpriteOffset.Y) + math.abs(Missile.SpriteOffset.Y)))
				Missile.Vel = UAV.Offset(self.Vel, self.angle, 5)
				Missile.RotAngle = self.angle
				Missile.AngularVel = self.AngularVel
				Missile:SetWhichMOToNotHit(self, 2.0)
				self:SetWhichMOToNotHit(Missile, 2.0)
				
				if MovableMan:ValidMO(self.RadarTarget) then
					Missile.Sharpness = self.RadarTarget.ID
				end
				
				MovableMan:AddParticle(Missile)
				
				self.payload = self.payload - 1
				self:AddInventoryItem(CreateHDFirearm(".", "Air Support.rte"))	-- negative gold
				
				-- Draw Missiles
				if self.payload > 2 then
					self.Frame = 0
				elseif self.payload > 0 then
					self.Frame = 1
				else
					self.Frame = 2
				end
			else
				UAV.AddIcon(self, CreateMOSParticle("No Ammo"), 5)
			end
		end
		
		-- Radar: Look for crafts
		if self.RadarTimer:IsPastSimMS(1000) then
			self.RadarTimer:Reset()
			self.EnemyCraft = {}
			self.FriendlyCraft = {}
			for Act in MovableMan.Actors do
				if (Act.ClassName == "ACRocket" or Act.ClassName == "ACDropShip" or Act.ClassName == "ACraft") and not(Act:IsStatus(Actor.DYING) or Act:IsDead()) then
					if Act.Team ~= self.Team then
						table.insert(self.EnemyCraft, Act)
					elseif Act.ID ~= self.ID then
						table.insert(self.FriendlyCraft, Act)
					end
				end
			end
		end
		
		-- Draw dots
		if SceneMan.SceneWrapsX then
			local startDot = nil
			self.drawEnemy = not self.drawEnemy
			if self.drawEnemy then
				local bestAngle = self.radarCone
				local ang = self.radarCone
				self.RadarTarget = nil
				for i = 1, #self.EnemyCraft do
					if MovableMan:ValidMO(self.EnemyCraft[i]) then
						ang = math.abs(SceneMan:ShortestDistance(self.Pos, self.EnemyCraft[i].Pos, true).AbsRadAngle - self.angle)
						if ang < self.radarCone then
							startDot = math.ceil((self.xDots/self.dotRes)*(self.EnemyCraft[i].Pos.X/SceneMan.SceneWidth))
							Dot = CreateMOPixel("EnemyPos", "Air Support.rte")
							Dot.Pos = Vector(self.GroundDots[startDot].Pos.X, self.Pos.Y + self.EnemyCraft[i].Pos.Y/self.yDiv + self.yDots + 20)
							Dot.Vel = self.Vel
							MovableMan:AddParticle(Dot)
							
							if ang < bestAngle then
								bestAngle = ang
								self.RadarTarget = self.EnemyCraft[i]
							end
						end
					else
						table.remove(self.EnemyCraft, i)
						break
					end
				end
			else
				for i = 1, #self.FriendlyCraft do
					if MovableMan:ValidMO(self.FriendlyCraft[i]) then
						if math.abs(SceneMan:ShortestDistance(self.Pos, self.FriendlyCraft[i].Pos, true).AbsRadAngle - self.angle) < self.radarCone then
							startDot = math.ceil((self.xDots/self.dotRes)*(self.FriendlyCraft[i].Pos.X/SceneMan.SceneWidth))
							Dot = CreateMOPixel("FriendlyPos", "Air Support.rte")
							Dot.Pos = Vector(self.GroundDots[startDot].Pos.X, self.Pos.Y + self.FriendlyCraft[i].Pos.Y/self.yDiv + self.yDots + 20)
							Dot.Vel = self.Vel
							MovableMan:AddParticle(Dot)
						end
					else
						table.remove(self.FriendlyCraft, i)
						break
					end
				end
			end
		else
			self.drawEnemy = not self.drawEnemy
			if self.drawEnemy then
				local bestAngle = self.radarCone
				local ang = self.radarCone
				self.RadarTarget = nil
				for i = 1, #self.EnemyCraft do
					if MovableMan:ValidMO(self.EnemyCraft[i]) then
						ang = math.abs((self.EnemyCraft[i].Pos - self.Pos).AbsRadAngle - self.angle)
						if ang < self.radarCone then
							Dot = CreateMOPixel("EnemyPos", "Air Support.rte")
							Dot.Pos = self.Pos + Vector(self.EnemyCraft[i].Pos.X/self.xDiv - self.xDots/2, self.EnemyCraft[i].Pos.Y/self.yDiv + self.yDots + 20)
							Dot.Vel = self.Vel
							MovableMan:AddParticle(Dot)
							
							if ang < bestAngle then
								bestAngle = ang
								self.RadarTarget = self.EnemyCraft[i]
							end
						end
					else
						table.remove(self.EnemyCraft, i)
						break
					end
				end
			else
				for i = 1, #self.FriendlyCraft do
					if MovableMan:ValidMO(self.FriendlyCraft[i]) then
						if math.abs((SceneMan:ShortestDistance(self.Pos, self.FriendlyCraft[i].Pos, true)).AbsRadAngle - self.angle) < self.radarCone then
							Dot = CreateMOPixel("FriendlyPos", "Air Support.rte")
							Dot.Pos = self.Pos + Vector(self.FriendlyCraft[i].Pos.X/self.xDiv - self.xDots/2, self.FriendlyCraft[i].Pos.Y/self.yDiv + self.yDots + 20)
							Dot.Vel = self.Vel
							MovableMan:AddParticle(Dot)
						end
					else
						table.remove(self.FriendlyCraft, i)
						break
					end
				end
			end
		end
		
		-- Draw targeting icons
		if self.TargetTimer:IsPastSimMS(100) and MovableMan:ValidMO(self.RadarTarget) then
			local ang = SceneMan:ShortestDistance(self.Pos, self.RadarTarget.Pos, true).AbsRadAngle
			
			local Dot = CreateMOSParticle("Radar Small", "Air Support.rte")
			Dot.Pos = UAV.Offset(self.Pos, ang, 70)
			Dot.Vel = self.Vel
			MovableMan:AddParticle(Dot)
			
			Dot = CreateMOSParticle("Radar Medium", "Air Support.rte")
			Dot.Pos = UAV.Offset(self.Pos, ang, 100)
			Dot.Vel = self.Vel
			MovableMan:AddParticle(Dot)
			
			Dot = CreateMOSParticle("Radar Large", "Air Support.rte")
			Dot.Pos = UAV.Offset(self.Pos, ang, 130)
			Dot.Vel = self.Vel
			MovableMan:AddParticle(Dot)
		end
	elseif self.playerCtrl then
		UAV.Destroy(self)
	end
end