function LandingPad:StartActivity()
	collectgarbage("collect")	-- Performs a full garbage-collection cycle
	
	-- Globals
	gLandingPad = {}
	gLandingPad.phase = 1
	gLandingPad.updates = {}
	
	-- Player brains
	for player = Activity.PLAYER_1, Activity.MAXPLAYERCOUNT - 1 do
		if self:PlayerActive(player) and self:PlayerHuman(player) then
			local Brain = CreateActor("Brain Case", "Base.rte")
			Brain.Team = Activity.TEAM_1
			Brain.Pos = Vector(480 + player * 16, 1028)
			MovableMan:AddActor(Brain)
			self:SetPlayerBrain(Brain, player)
			self:SetObservationTarget(Brain.Pos, player)
			self.activePlayer = player
		end
	end
	
	self.diffi = 0	-- a higher value increase spawn rate
	self.lives = 0	-- # of extra gunships
	
	if self.Difficulty <= GameActivity.CAKEDIFFICULTY then
		self.lives = 25
	elseif self.Difficulty <= GameActivity.EASYDIFFICULTY then
		self.lives = 6
	elseif self.Difficulty <= GameActivity.MEDIUMDIFFICULTY then
		self.lives = 3
		self.diffi = 2
	elseif self.Difficulty <= GameActivity.HARDDIFFICULTY then
		self.lives = 2
		self.diffi = 7
	elseif self.Difficulty <= GameActivity.NUTSDIFFICULTY then
		self.lives = 1
		self.diffi = 14
	else
		self.lives = 0
		self.diffi = 21
	end
	
	self:SetTeamFunds(0, Activity.TEAM_1)
	
	self.KeyTimer = Timer()
	self.UpdateTimer = Timer()
	self.TriggerTimer = Timer()
	self.updateTiming = 1000
	
	self.MortarArea = SceneMan.Scene:GetArea("Mortar")
	self.ObjectArea = SceneMan.Scene:GetArea("Area3")
	self.Teleports = {}
	self.SpawnAreas = {}
	self.Player = {}
	self.Enemy = {}
	self.SAM = {}
	
	self.Player.Actor = "RClone"
	self.Enemy.Actor = "CClone"
	
	-- Initialize player equipment
	local Wp = CreateHDFirearm("Uzi", "Ronin.rte")	-- Check if the rte is avaliable
	if Wp then
		Wp.Pos = Vector(-50, 4000)
		Wp.ToDelete = true
		MovableMan:AddParticle(Wp)
		
		self.Player.rte = "Ronin.rte"
		self.Player.Weapons = {"Pumpgun", "Spas 12", "M16", "M16", "AK-47", "AK-47", "M1 Garand", "M1 Garand"}
		self.Player.LongRange = {"M16", "AK-47", "M60", "M1 Garand", "Kar98", "Kar98"}
		self.Player.Sidearms = {"Glock", "Glock", "Uzi", "Desert Eagle", "Peacemaker", "Shortgun"}
		self.Player.Grenades = {"Pineapple Grenade", "Pineapple Grenade", "Stick Grenade"}
	else
		self.Player.rte = "Base.rte"
		self.Player.Weapons = {"SMG", "SMG", "Shotgun"}
		self.Player.LongRange = {"SMG"}
		self.Player.Sidearms = {"Blaster Pistol"}
		self.Player.Grenades = {"Frag Grenade"}
	end
	
	-- Initialize enemy equipment
	self.Enemy.Assault = {"Shotgun.", "Carbine.", "Carbine.", "Assault Rifle.", "Auto Shotgun.", "Auto Shotgun.", "Napalm Flamer."}
	
	if self.lives > 2 then
		self.Enemy.Weapons = {"Assault Rifle.", "Assault Rifle.", "Shotgun.", "Auto Shotgun.", "Rocket Launcher.", "Sniper Rifle.", 
							  "Auto Cannon.", "Flak Cannon.", "Revolver Cannon."}
		self.Enemy.LongRange = {"Assault Rifle.", "Assault Rifle.", "Assault Rifle.", "Sniper Rifle.", "Heavy Sniper Rifle.",
								"Heavy Sniper Rifle.", "Rocket Launcher.", "Homing Missile Launcher.", "Flak Cannon.", "Flak Cannon."}
	else
		self.Enemy.Weapons = {"Assault Rifle.", "Assault Rifle.", "Shotgun.", "Auto Shotgun.", "Rocket Launcher.", "Sniper Rifle.", 
							  "Homing Missile Launcher.", "Auto Cannon.", "Flak Cannon.", "Revolver Cannon."}
		self.Enemy.LongRange = {"Assault Rifle.", "Sniper Rifle.", "Heavy Sniper Rifle.", "Heavy Sniper Rifle.", "Rocket Launcher.",
							"Homing Missile Launcher.", "Homing Missile Launcher.", "Homing Missile Launcher.", "Flak Cannon.", 
							"Flak Cannon.", "Flak Cannon."}
	end
	
	self.Enemy.Sidearms = {"Auto Pistol.", "Pistol.", "Pistol."}
	self.Enemy.Grenades = {"Grenade"}
		
	for i = 1, 6 do
		self.SpawnAreas[i] = SceneMan.Scene:GetArea("SpawnArea" .. i)
		if i < 4 then
			self.Teleports[i] = CreateMOSRotating("Teleport", "Air Support.rte")
			self.Teleports[i].Pos = self.SpawnAreas[i]:GetCenterPoint() + Vector(0, -52)
			MovableMan:AddParticle(self.Teleports[i])
		end
	end
	
	local Ar = nil
	for i = 1, 5 do
		Ar = SceneMan.Scene:GetArea("EGL" .. i)
		if Ar then
			Act = self:CreateEnemyLongRange(Actor.AIMODE_SENTRY)
			Act.Pos = Ar:GetCenterPoint()
			MovableMan:AddActor(Act)
		end
	end
	
	for i = 1, 3 do
		Ar = SceneMan.Scene:GetArea("AGR" .. i)
		if Ar then
			Act = self:CreatePlayerLongRange(Actor.AIMODE_SENTRY)
			Act.Pos = Ar:GetCenterPoint()
			MovableMan:AddActor(Act)
		end
	end
	
	-- Target
	self.Container = CreateAEmitter("Container", "Air Support.rte")
	if self.Container then
		self.Container.Pos = SceneMan.Scene:GetArea("Area3"):GetCenterPoint() + Vector(0,-88)
		MovableMan:AddParticle(self.Container)
	end
	
	-- Spawn the mortar
	self.Mortar = CreateActor("Mortar", "Air Support.rte")
	if self.Mortar then
		self.Mortar.Sharpness = 1100	-- min range
		self.Mortar.Pos = SceneMan:MovePointToGround(Vector(self.MortarArea:GetCenterPoint().X, 0), 16, 2)
		self.Mortar.Team = Activity.TEAM_2
		MovableMan:AddActor(self.Mortar)
	end
	
	-- Assault team (lambs for the slaughter)
	self.AssaultTeam = {}
	self.AssaultTeam[1] = self:CreatePlayer(Actor.AIMODE_GOTO)
	if self.AssaultTeam[1] then
		self.AssaultTeam[1].Pos = Vector(1650, 960)
		self.AssaultTeam[1]:AddAISceneWaypoint(self.SpawnAreas[1]:GetCenterPoint())
		MovableMan:AddActor(self.AssaultTeam[1])
	end
	
	self.AssaultTeam[2] = self:CreatePlayer(Actor.AIMODE_GOTO)
	if self.AssaultTeam[2] then
		self.AssaultTeam[2].Pos = Vector(1550, 970)
		self.AssaultTeam[2]:AddAISceneWaypoint(self.SpawnAreas[2]:GetCenterPoint())
		MovableMan:AddActor(self.AssaultTeam[2])
	end
	
	self.AssaultTeam[3] = self:CreatePlayer(Actor.AIMODE_GOTO)
	if self.AssaultTeam[3] then
		self.AssaultTeam[3].Pos = Vector(1450, 1000)
		self.AssaultTeam[3]:AddAISceneWaypoint(self.SpawnAreas[3]:GetCenterPoint())
		MovableMan:AddActor(self.AssaultTeam[3])
		self:SwitchToActor(ToActor(self.AssaultTeam[3]), self.activePlayer, self.AssaultTeam[3].Team)
	end
	
	-- Triggers
	self.Triggers = {}
	self:AddTrigger(GarbageCollect, 500, 5000)	-- Performs a garbage-collection step
end

function LandingPad:Dead(MO)
	if MO and (MovableMan:IsActor(MO) or MovableMan:IsParticle(MO)) and MO.ClassName ~= "Entity" then
		return false
	end
	
	return true
end

function LandingPad:CreatePlayer(Mode, flipped)
	local Act = CreateAHuman(self.Player.Actor, "Air Support.rte")
	Act.Team = Activity.TEAM_1
	Act.HFlipped = flipped or false
	Act.AIMode = Mode or Actor.AIMODE_SENTRY
	Act:AddInventoryItem(CreateHDFirearm(self.Player.Weapons[SelectRand(1, #self.Player.Weapons+1)], self.Player.rte))
	Act:AddInventoryItem(CreateHDFirearm(self.Player.Sidearms[SelectRand(1, #self.Player.Sidearms+1)], self.Player.rte))
	Act:AddInventoryItem(CreateTDExplosive(self.Player.Grenades[SelectRand(1, #self.Player.Grenades+1)], self.Player.rte))
	
	if PosRand() < 0.3 then
		Act:AddInventoryItem(CreateTDExplosive(self.Player.Grenades[SelectRand(1, #self.Player.Grenades+1)], self.Player.rte))
	end
	
	return Act
end

function LandingPad:CreatePlayerLongRange(Mode, flipped)
	local Act = CreateAHuman(self.Player.Actor, "Air Support.rte")
	Act.Team = Activity.TEAM_1
	Act.HFlipped = flipped or false
	Act.AIMode = Mode or Actor.AIMODE_SENTRY
	Act:AddInventoryItem(CreateHDFirearm(self.Player.LongRange[SelectRand(1, #self.Player.LongRange+1)], self.Player.rte))
	Act:AddInventoryItem(CreateHDFirearm(self.Player.Sidearms[SelectRand(1, #self.Player.Sidearms+1)], self.Player.rte))
	
	if PosRand() < 0.2 then
		Act:AddInventoryItem(CreateTDExplosive(self.Player.Grenades[SelectRand(1, #self.Player.Grenades+1)], self.Player.rte))
	end
	
	return Act
end

function LandingPad:CreateEnemy(Mode, flipped)
	local Act = CreateAHuman(self.Enemy.Actor, "Air Support.rte")
	Act.Team = Activity.TEAM_2
	Act.HFlipped = flipped or true
	Act.AIMode = Mode or Actor.AIMODE_PATROL
	Act:AddInventoryItem(CreateHDFirearm(self.Enemy.Weapons[SelectRand(1, #self.Enemy.Weapons+1)], "Air Support.rte"))
	Act:AddInventoryItem(CreateHDFirearm(self.Enemy.Sidearms[SelectRand(1, #self.Enemy.Sidearms+1)], "Air Support.rte"))
	
	if PosRand() < 0.2 then
		Act:AddInventoryItem(CreateTDExplosive(self.Enemy.Grenades[SelectRand(1, #self.Enemy.Grenades+1)], "Coalition.rte"))
	end
	
	return Act
end

function LandingPad:CreateEnemyLongRange(Mode, flipped)
	local Act = CreateAHuman(self.Enemy.Actor, "Air Support.rte")
	Act.Team = Activity.TEAM_2
	Act.HFlipped = flipped or true
	Act.AIMode = Mode or Actor.AIMODE_PATROL
	Act:AddInventoryItem(CreateHDFirearm(self.Enemy.LongRange[SelectRand(1, #self.Enemy.LongRange+1)], "Air Support.rte"))
	Act:AddInventoryItem(CreateHDFirearm(self.Enemy.Sidearms[SelectRand(1, #self.Enemy.Sidearms+1)], "Air Support.rte"))
	
	return Act
end

function LandingPad:CreateFlyingEnemy(Mode, Wpt)
	local Act = CreateAHuman("Soldier Light", "Coalition.rte")
	if Act then
		if Wpt then
			Act:AddAISceneWaypoint(Wpt)
		end
		
		Act.Team = Activity.TEAM_2
		Act.HFlipped = flipped or true
		Act.AIMode = Actor.AIMODE_GOTO
		Act.Sharpness = Mode or Actor.AIMODE_SENTRY
		Act:AddInventoryItem(CreateHDFirearm(self.Enemy.Assault[SelectRand(1, #self.Enemy.Assault+1)], "Air Support.rte"))
		Act:AddInventoryItem(CreateHDFirearm(self.Enemy.Sidearms[SelectRand(1, #self.Enemy.Sidearms+1)], "Air Support.rte"))
		
		-- print(Act.PresetName .. " reload = " .. Act:ReloadScripts())	-- Debug
	end
	
	return Act
end

function LandingPad:CreateFlyingSentry(Mode, Wpt)
	local Act = CreateAHuman("Soldier Light", "Coalition.rte")
	if Act then
		if Wpt then
			Act:AddAISceneWaypoint(Wpt)
		end
		
		Act.Team = Activity.TEAM_2
		Act.HFlipped = flipped or true
		Act.AIMode = Actor.AIMODE_GOTO
		Act.Sharpness = Mode or Actor.AIMODE_SENTRY
		Act:AddInventoryItem(CreateHDFirearm(self.Enemy.LongRange[SelectRand(1, #self.Enemy.LongRange+1)], "Air Support.rte"))
		Act:AddInventoryItem(CreateHDFirearm(self.Enemy.Sidearms[SelectRand(1, #self.Enemy.Sidearms+1)], "Air Support.rte"))
		
		-- print(Act.PresetName .. " reload = " .. Act:ReloadScripts())	-- Debug
	end
	
	return Act
end

function LandingPad:UpdateActivity()
	if self.ActivityState == Activity.OVER then
		return
	end
	
	self:ClearObjectivePoints()
	
	if self.TriggerTimer:IsPastSimMS(400) then
		local deleteIndex
		self.TriggerTimer:Reset()
		self:SetTeamFunds(0, Activity.TEAM_1)
		
		for i,Trg in ipairs(self.Triggers) do
			if Trg.Timer:IsPastSimMS(Trg.timerLimit or 0) then
				Trg.Timer:Reset()
				Trg.Result(self, Trg)
				Trg.timerLimit = Trg.nextTimerLimit
				
				if Trg.delete then
					deleteIndex = i
				end
				
				break	-- only one trigger per frame
			end
		end
		
		if deleteIndex then
			table.remove(self.Triggers, deleteIndex)
		end
	end
	
	if self.UpdateTimer and self.UpdateTimer:IsPastSimMS(self.updateTiming) then
		self.UpdateTimer:Reset()
		
		if gLandingPad.phase == 1 then	-- Send in some Ronin fodder
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.Marker = nil	-- TODO: Attack target
				self.Msg = nil
				self.updateTiming = 1000
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self.Msg = "Forward!"
				self.updateTiming = 5000
			end
			
			for i = 1, #self.AssaultTeam do
				if not self:Dead(self.AssaultTeam[i]) then
					break
				else
					self.AssaultTeam[i] = nil
				end
			end
			
			if gLandingPad.updates[gLandingPad.phase] > 21 or #self.AssaultTeam < 1 then
				gLandingPad.phase = gLandingPad.phase + 1
				self.updateTiming = 500
			end
		elseif gLandingPad.phase == 2 then	-- Land rockets
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.Msg = nil
				self.updateTiming = 1000 + self.diffi*100
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(RocketAttack, 8000)
				self:AddTrigger(ShortRangeAttack, 4000)
				self.Msg = "Enemy counter attack!"
				self.updateTiming = 5000
			end
			
			if gLandingPad.updates[gLandingPad.phase] == 3 then
				self.Msg = "Wait for air support"
				self.updateTiming = 5000
			end
			
			if gLandingPad.updates[gLandingPad.phase] == 27 then
				self.Reinforce = CreateACRocket("Rocket MK1a", "Air Support.rte")
				if self.Reinforce then
					self.Reinforce.Pos = Vector(SceneMan.Scene:GetArea("LZ Team 1"):GetCenterPoint().X,-50)
					self.Reinforce.Vel = Vector(0,-1)
					self.Reinforce.Team = Activity.TEAM_1
					self.Reinforce:AddInventoryItem(self:CreatePlayer(Actor.AIMODE_SENTRY))
					--MovableMan:AddActor(self.Reinforce)
					MovableMan:AddParticle(self.Reinforce)
				end
			end
			
			if gLandingPad.updates[gLandingPad.phase] > 52 then
				gLandingPad.phase = gLandingPad.phase + 1
			end
		elseif gLandingPad.phase == 3 then	-- Spawn the first gunship
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.updateTiming = 500
				self.Msg = nil
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(LongRangeAttack, 4000)
				self.Msg = "Repel the enemy attack"
				self.updateTiming = 5000
				
				self.Gunship = CreateACDropShip("Hydra Gunship", "Air Support.rte")
				if self.Gunship then
					self.Gunship.Pos = Vector(100, -50)
					self.Gunship.Vel = Vector(0, 15)
					self.Gunship.Team = Activity.TEAM_1
					MovableMan:AddActor(self.Gunship)
					self:SwitchToActor(ToActor(self.Gunship), self.activePlayer, self.Gunship.Team)
				end
			end
			
			if gLandingPad.updates[gLandingPad.phase] == 10 then
				self.Reinforce = CreateACRocket("Rocket MK1a", "Air Support.rte")
				if self.Reinforce then
					self.Reinforce.Pos = Vector(SceneMan.Scene:GetArea("LZ Team 1"):GetCenterPoint().X+200,-50)
					self.Reinforce.Vel = Vector(0,5)
					self.Reinforce.Team = Activity.TEAM_1
					self.Reinforce:AddInventoryItem(self:CreatePlayer(Actor.AIMODE_SENTRY))
					--MovableMan:AddActor(self.Reinforce)
					MovableMan:AddParticle(self.Reinforce)
				end
			end
			
			if self:Dead(self.Gunship) then
				if self.lives > 0 then
					self.lives = self.lives - 1
					
					self.Gunship = CreateACDropShip("Hydra Gunship", "Air Support.rte")
					if self.Gunship then
						self.Gunship.Pos = Vector(100, -50)
						self.Gunship.Vel = Vector(0, 15)
						self.Gunship.Team = Activity.TEAM_1
						MovableMan:AddActor(self.Gunship)
						self:SwitchToActor(ToActor(self.Gunship), self.activePlayer, self.Gunship.Team)
					end
				else
					gLandingPad.phase = 0
				end
			elseif self.Gunship.Pos.X > 1800 then
				gLandingPad.phase = gLandingPad.phase + 1
			end
		elseif gLandingPad.phase == 4 then	-- Increase the AA capability
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.updateTiming = 1000
				self.Msg = nil
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(SpawnAA, 2000 - self.diffi*50)	-- Spawn enemies at the right edge of the map
				self.Marker = self.Mortar
				self.Msg = "Destroy the enemy artillery"
				self.updateTiming = 5000
				
				local Act = CreateACRocket("SAM Site Drop", "Air Support.rte")
				if Act then
					Act.Pos = Vector(SceneMan.Scene:GetArea("SAM1"):GetCenterPoint().X, -50)
					Act.Vel = Vector(0,10)
					Act.Team = Activity.TEAM_2
					MovableMan:AddActor(Act)
					self.SAM[1] = Act
				end
			end
			
			if self:Dead(self.Gunship) then
				if self.lives > 0 then
					self.lives = self.lives - 1
					
					self.Gunship = CreateACDropShip("Hydra Gunship", "Air Support.rte")
					if self.Gunship then
						self.Gunship.Pos = Vector(100, -50)
						self.Gunship.Vel = Vector(0, 15)
						self.Gunship.Team = Activity.TEAM_1
						MovableMan:AddActor(self.Gunship)
						self:SwitchToActor(ToActor(self.Gunship), self.activePlayer, self.Gunship.Team)
					end
				else
					gLandingPad.phase = 0
				end
			elseif gLandingPad.updates[gLandingPad.phase] > SelectRand(80,120) or self:Dead(self.Mortar) then
				gLandingPad.phase = gLandingPad.phase + 1
			end
		elseif gLandingPad.phase == 5 then	-- Increase the AA capability again
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				
				local Act = CreateACRocket("SAM Site Drop", "Air Support.rte")
				if Act then
					Act.Pos = Vector(SceneMan.Scene:GetArea("SAM2"):GetCenterPoint().X, -50)
					Act.Vel = Vector(0,10)
					Act.Team = Activity.TEAM_2
					MovableMan:AddActor(Act)
					self.SAM[2] = Act
				end
				
				self:AddTrigger(SpawnSentry2)	-- Spawn sentry at teleport 2
				self.updateTiming = 1000
			end
			
			if self:Dead(self.Gunship) then
				if self.lives > 0 then
					self.lives = self.lives - 1
					
					self.Gunship = CreateACDropShip("Hydra Gunship", "Air Support.rte")
					if self.Gunship then
						self.Gunship.Pos = Vector(100, -50)
						self.Gunship.Vel = Vector(0, 15)
						self.Gunship.Team = Activity.TEAM_1
						MovableMan:AddActor(self.Gunship)
						self:SwitchToActor(ToActor(self.Gunship), self.activePlayer, self.Gunship.Team)
					end
				else
					gLandingPad.phase = 0
				end
			elseif self:Dead(self.Mortar) then
				gLandingPad.phase = gLandingPad.phase + 1
			end
		elseif gLandingPad.phase == 6 then	-- Mortar destroyed
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				
				local Act = CreateACRocket("SAM Site Drop", "Air Support.rte")
				if Act then
					if self:Dead(self.SAM[2]) then
						Act.Pos = Vector(SceneMan.Scene:GetArea("SAM2"):GetCenterPoint().X, -50)
					elseif self:Dead(self.SAM[1]) then
						Act.Pos = Vector(SceneMan.Scene:GetArea("SAM1"):GetCenterPoint().X, -50)
					else
						Act.Pos = Vector(SceneMan.Scene:GetArea("SAM3"):GetCenterPoint().X, -50)
					end
					
					Act.Vel = Vector(0,10)
					Act.Team = Activity.TEAM_2
					MovableMan:AddActor(Act)
					self.SAM[3] = Act
				end
				
				self:AddTrigger(SpawnDefence)	-- Spawn defence at teleport 1
				self.updateTiming = 1000
			end
			
			if self:Dead(self.Gunship) then
				gLandingPad.phase = 7
			elseif gLandingPad.updates[gLandingPad.phase] > 10 then
				gLandingPad.phase = 9
			end
		elseif gLandingPad.phase == 7 then	-- Spawn basilisk gunship
			self.Gunship = CreateACDropShip("Basilisk Gunship", "Air Support.rte")
			if self.Gunship then
				self.Gunship.Pos = Vector(100, -50)
				self.Gunship.Vel = Vector(0, 15)
				self.Gunship.Team = Activity.TEAM_1
				MovableMan:AddActor(self.Gunship)
				self:SwitchToActor(ToActor(self.Gunship), self.activePlayer, self.Gunship.Team)
			end
			
			gLandingPad.phase = 8
		elseif gLandingPad.phase == 8 then	-- Another airborne assault, this time by dropship
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.Msg = nil
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(DropShipAttack, 8000)
				self.Msg = "Incomming drop ships"
				self.updateTiming = 5000
			end
			
			if self:Dead(self.Gunship) then
				if self.lives > 0 then
					self.lives = self.lives - 1
					
					self.Gunship = CreateACDropShip("Basilisk Gunship", "Air Support.rte")
					if self.Gunship then
						self.Gunship.Pos = Vector(100, -50)
						self.Gunship.Vel = Vector(0, 15)
						self.Gunship.Team = Activity.TEAM_1
						MovableMan:AddActor(self.Gunship)
						self:SwitchToActor(ToActor(self.Gunship), self.activePlayer, self.Gunship.Team)
					end
				else
					gLandingPad.phase = 0
				end
			elseif gLandingPad.updates[gLandingPad.phase] > SelectRand(11,17) then
				gLandingPad.phase = 10
			end
		elseif gLandingPad.phase == 9 then	-- Ronin counter attack, hydra
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.updateTiming = 1000
				self.Msg = nil
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self.Msg = "Infiltrate enemy base"
				self:AddTrigger(SpawnRonin)
				self.updateTiming = 5000
			end
			
			if self:Dead(self.Gunship) then
				gLandingPad.phase = 7
			elseif self:Dead(self.Teleports[1]) then
				gLandingPad.phase = 11
			else
				for Act in MovableMan.Actors do
					if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase1", Act.Pos) then
						gLandingPad.phase = 12
						break
					end
				end
			end
		elseif gLandingPad.phase == 10 then	-- Ronin counter attack, basilisk
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
				self.updateTiming = 1000
				self.Msg = nil
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self.Msg = "Infiltrate enemy base"
				self:AddTrigger(SpawnRonin)
				self.updateTiming = 5000
			end
			
			if self:Dead(self.Teleports[1]) then
				gLandingPad.phase = 11
			else
				for Act in MovableMan.Actors do
					if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase1", Act.Pos) then
						gLandingPad.phase = 12
						break
					end
				end
			end
		elseif gLandingPad.phase == 11 then	-- Ronin land inside enemy base
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(SpawnRonin2)
				self.updateTiming = 1000
			end
			
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase1", Act.Pos) then
					gLandingPad.phase = 12
					break
				end
			end
		elseif gLandingPad.phase == 12 then	-- Ronin inside 1
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(SpawnPatrol1)
				self.updateTiming = 1000
			end
			
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase2", Act.Pos) then
					gLandingPad.phase = 13
					break
				end
			end
		elseif gLandingPad.phase == 13 then	-- Ronin inside 2
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(SpawnPatrol2)
				self:AddTrigger(RocketDefence, 7000)
				self.updateTiming = 1000
			end
			
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase3", Act.Pos) then
					gLandingPad.phase = 14
					break
				end
			end
		elseif gLandingPad.phase == 14 then	-- Ronin inside 3
			if gLandingPad.updates[gLandingPad.phase] then
				gLandingPad.updates[gLandingPad.phase] = gLandingPad.updates[gLandingPad.phase] + 1
			else
				gLandingPad.updates[gLandingPad.phase] = 1
				self:AddTrigger(SpawnSentry3)
				self.updateTiming = 5000
			end
		elseif gLandingPad.phase == 0 then
			ActivityMan:EndActivity()
		else
			-- print("Phase " .. gLandingPad.phase .. " End")
			self.updateTiming = 9999999
		end
		--[[
		if gLandingPad.updates[gLandingPad.phase] then
			print("Phase " .. gLandingPad.phase .. " update " .. gLandingPad.updates[gLandingPad.phase])
		end]]
	end
	
	if self.Msg then
		FrameMan:ClearScreenText(Activity.TEAM_1)
		FrameMan:SetScreenText(self.Msg, Activity.TEAM_1, 0, 4000, true)
	end
	
	if self.Marker then
		if self:Dead(self.Marker) then
			self.Marker = nil
		else
			self:AddObjectivePoint("Destroy", self.Marker.AboveHUDPos, Activity.TEAM_1, GameActivity.ARROWDOWN)
		end
	end
	
	if self.Reinforce then
		if self:Dead(self.Reinforce) then
			self.Reinforce = nil
		else
			self:AddObjectivePoint("Reinforcements", self.Reinforce.Pos + Vector(0, -70), Activity.TEAM_1, GameActivity.ARROWDOWN)
		end
	end
	
	if self.Retrieve then
		if self:Dead(self.Retrieve) then
			self.Retrieve = nil
		else
			self:AddObjectivePoint("Retrieve", self.Retrieve.AboveHUDPos, Activity.TEAM_1, GameActivity.ARROWDOWN)
		end
	end
	
	if self:Dead(self.Container) then
		self.WinnerTeam = Activity.TEAM_1
		ActivityMan:EndActivity()
	end
	
	--[[ Debug spawn
	if self.KeyTimer:IsPastSimMS(250) and UInputMan:KeyHeld(71) then
		if UInputMan:KeyHeld(6) then	-- F
			self.KeyTimer:Reset()
			local Ship = CreateACDropShip("Basilisk Gunship", "Air Support.rte")
			if Ship then
				Ship.Pos = Vector(180, 500)
				Ship.Team = Activity.TEAM_1
				Ship.AIMode = Actor.AIMODE_STAY
				print(Ship.PresetName .. " reload = " .. Ship:ReloadScripts())
				MovableMan:AddActor(Ship)
				self:SwitchToActor(ToActor(Ship), Ship.Team, Ship.Team)
			end
		elseif UInputMan:KeyHeld(7) then	-- G
			self.KeyTimer:Reset()
			local Ship = CreateACDropShip("Hydra Gunship", "Air Support.rte")
			if Ship then
				Ship.Pos = Vector(180, 400)
				Ship.Team = Activity.TEAM_1
				Ship.AIMode = Actor.AIMODE_STAY
				print(Ship.PresetName .. " reload = " .. Ship:ReloadScripts())
				MovableMan:AddActor(Ship)
				self:SwitchToActor(ToActor(Ship), Ship.Team, Ship.Team)
			end
		elseif UInputMan:KeyHeld(8) then	-- H
			self.KeyTimer:Reset()
			local Act = CreateACRocket("Mortar Drop", "Air Support.rte")
			if Act then
				Act.Pos = Vector(400, -50)
				Act.Vel = Vector(-100, -100) --Act.Vel = Vector(0,10)
				print(Act.PresetName .. " script: " .. Act:ReloadScripts())
				MovableMan:AddActor(Act)
				self:SwitchToActor(ToActor(Act), Act.Team, Act.Team)
				
				local Emitter = CreateAEmitter("SAM", "Air Support.rte")
				if Emitter then
					Emitter.Pos = Vector(4000,0)
					print(Emitter.PresetName .. " script: " .. Emitter:ReloadScripts())
					MovableMan:AddParticle(Emitter)
				end
			end
		elseif UInputMan:KeyHeld(10) then	-- J
			self.KeyTimer:Reset()
			local Act = CreateActor("Mortar", "Air Support.rte")
			if Act then
				print(Act.PresetName .. " script: " .. Act:ReloadScripts())
				Act.Pos = SceneMan:MovePointToGround(Vector(700, 0), 16, 2)
				Act.Team = Activity.TEAM_1
				MovableMan:AddActor(Act)
				
				local Bomb = CreateAEmitter("Mortar Bomb", "Air Support.rte")
				if Bomb then
					print(Bomb.PresetName .. " script: " .. Bomb:ReloadScripts())
					Bomb.ToDelete = true
					MovableMan:AddParticle(Bomb)
				end
			end
		elseif UInputMan:KeyHeld(15) then -- O
			self.KeyTimer:Reset()
			local Act = self:CreatePlayer()
			if Act then
				Act.Pos = SceneMan.Scene:GetArea("Area3"):GetCenterPoint()
				print(Act.PresetName .. " script: " .. Act:ReloadScripts())
				MovableMan:AddActor(Act)
				self:SwitchToActor(ToActor(Act), Act.Team, Act.Team)
			end
		elseif UInputMan:KeyHeld(22) then	-- V
			self.KeyTimer:Reset()
			self:SetViewState(Activity.ACTORSELECT, Activity.TEAM_1)
			self:SetActorSelectCursor(Vector(3500, 600), Activity.TEAM_1)
		elseif UInputMan:KeyHeld(71) and UInputMan:KeyHeld(26) then
			self.KeyTimer:Reset()
			for Act in MovableMan.Actors do
				if Act.PresetName ~= "Brain Robot" and Act.PresetName ~= "Brain Case" and Act.ClassName ~= "ADoor" then
					Act.ToDelete = true
				end
			end
		elseif UInputMan:KeyHeld(3) then	-- C
			self.KeyTimer:Reset()
			for Act in MovableMan.Actors do
				if Act.Team ~= 0 and Act.ClassName == "AHuman" then
					Act.ToDelete = true
				end
			end
		end
	end]]
end

----------------------------------------
--------------- Triggers ---------------
----------------------------------------
function LandingPad:AddTrigger(Function, nextLimit, firstLimit)
	table.insert(self.Triggers, {Result = Function, Timer = Timer(), nextTimerLimit = nextLimit or 1000, timerLimit = firstLimit or 0})
end

function GarbageCollect(self, Trigger)
	collectgarbage("step")
end

function RocketAttack(self, Trigger)
	if gLandingPad.phase > 3 or (gLandingPad.updates[3] and gLandingPad.updates[3] > 40) then
		Trigger.delete = true
	else
		-- Stop if the base is undefended
		local spawn = false
		for Act in MovableMan.Actors do
			if Act.Team == Activity.TEAM_1 and Act.ClassName == "AHuman" then
				spawn = true
				break
			end
		end
		
		if spawn then
			Trigger.nextTimerLimit = Trigger.nextTimerLimit + SelectRand(400+self.lives*100, 1600) + self.lives*500
			
			for i = 1, SelectRand(2,4) do	-- land 2-3 rockets
				if MovableMan:GetMOIDCount() < 170 + self.diffi then
					local Rocket = CreateACRocket("Rocket MK3", "Air Support.rte")
					if Rocket then
						local Pos = SceneMan.Scene:GetArea("Rocket" .. i):GetCenterPoint()
						Rocket.Pos = Vector(Pos.X,-50)
						Rocket.Vel = Vector(0,30)
						Rocket.Team = Activity.TEAM_2
						if PosRand() < 0.4 + self.lives*0.08 then
							Rocket:AddInventoryItem(self:CreateEnemy(Actor.AIMODE_BRAINHUNT))
						else
							Rocket:AddInventoryItem(self:CreateFlyingEnemy(Actor.AIMODE_BRAINHUNT, SceneMan.Scene:GetArea("AGR1"):GetCenterPoint()+Vector(-100,0)))
						end
						
						if PosRand() < (0.6 + self.diffi/50 - self.lives*0.12) then
							Rocket:AddInventoryItem(self:CreateEnemyLongRange(Actor.AIMODE_BRAINHUNT))
						end
						
						MovableMan:AddActor(Rocket)
					end
				end
			end
		end
	end
end

function RocketDefence(self, Trigger)
	if gLandingPad.phase > 15 then
		Trigger.delete = true
	else
		local spawn = false
		for Act in MovableMan.Actors do
			if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase4", Act.Pos) then
				spawn = true
				break
			end
		end
		
		if spawn then
			Trigger.nextTimerLimit = Trigger.nextTimerLimit + SelectRand(300, 900) + self.lives*550
			
			for i = 4, 5 do	-- land 2 rockets
				if MovableMan:GetMOIDCount() < 160 + self.diffi then
					local Rocket = CreateACRocket("Rocket MK3", "Air Support.rte")
					if Rocket then
						local Pos = SceneMan.Scene:GetArea("Rocket" .. i):GetCenterPoint()
						Rocket.Pos = Vector(Pos.X,-50)
						Rocket.Vel = Vector(0,5)
						Rocket.Team = Activity.TEAM_2
						
						local Act = self:CreateEnemy(Actor.AIMODE_GOTO)
						if Act then
							Act:AddAISceneWaypoint(SceneMan.Scene:GetArea("Area3"):GetCenterPoint())
							Rocket:AddInventoryItem(Act)
						end
						
						if PosRand() < (0.6 + self.diffi/50 - self.lives*0.1) then
							Act = self:CreateEnemyLongRange(Actor.AIMODE_GOTO)
							if Act then
								Act:AddAISceneWaypoint(SceneMan.Scene:GetArea("Area3"):GetCenterPoint())
								Rocket:AddInventoryItem(Act)
							end
						end
						
						MovableMan:AddActor(Rocket)
					end
				end
			end
		end
	end
end

function DropShipAttack(self, Trigger)
	if gLandingPad.phase > 8 then
		Trigger.delete = true
	else
		Trigger.nextTimerLimit = SelectRand(8000, 12000) + self.lives*300
		if MovableMan:GetMOIDCount() < 160 + self.diffi then
			local Ship = CreateACDropShip("Drop Ship MK1a", "Air Support.rte")
			if Ship then
				local index = SelectRand(1, 4)
				local Pos = SceneMan.Scene:GetArea("Rocket" .. index):GetCenterPoint()
				Ship.Pos = Vector(Pos.X, -50)
				Ship.Vel = Vector(-2*index-15, 22)
				Ship.Team = Activity.TEAM_2
				
				if PosRand() < (0.65 + self.diffi/50) then
					Ship:AddInventoryItem(self:CreateEnemy(Actor.AIMODE_BRAINHUNT))
					Ship:AddInventoryItem(self:CreateEnemyLongRange(Actor.AIMODE_BRAINHUNT))
				else
					Ship:AddInventoryItem(self:CreateEnemy(Actor.AIMODE_BRAINHUNT))
				end
				
				MovableMan:AddActor(Ship)
			end
		end
	end
end

function ShortRangeAttack(self, Trigger)
	if gLandingPad.phase > 2 or (gLandingPad.updates[2] and gLandingPad.updates[2] > 20) then
		Trigger.delete = true
	elseif MovableMan:GetMOIDCount() < 140 + self.diffi then
		local index = 1
		if MovableMan:ValidMO(self.Teleports[index]) and self.Teleports[index].ClassName ~= "Entity" then
			local spawn = true
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("SpawnArea" .. index, Act.Pos) then
					spawn = false
					break
				end
			end
			
			if spawn then
				Trigger.nextTimerLimit = 3000 - self.diffi*10 + self.lives*500
				
				local Act = nil
				if PosRand() < 0.95 then
					Act = self:CreateEnemy(Actor.AIMODE_GOTO)
				else
					Act = CreateACrab("AA Crawler", "Air Support.rte")
					if Act then
						Act.Team = Activity.TEAM_2
						Act.AIMode = Actor.AIMODE_GOTO
					end
				end
				
				if Act then
					Act:AddAISceneWaypoint(SceneMan.Scene:GetArea("Wpt1"):GetCenterPoint())
					Act:AddAISceneWaypoint(Vector(490, 1050))	-- Player brain
					Act:FlashWhite(200)
					Act.Pos = self.SpawnAreas[index]:GetCenterPoint()
					MovableMan:AddActor(Act)
					
					local Fx = CreateAEmitter("Teleporter Effect B")
					Fx.Pos = Act.Pos
					MovableMan:AddParticle(Fx)
				end
			else
				Trigger.nextTimerLimit = 1000
			end
		else
			Trigger.delete = true		-- The teleport was destroyed
		end
	end
end

function LongRangeAttack(self, Trigger)
	if gLandingPad.phase > 4 then
		Trigger.delete = true
	elseif MovableMan:GetMOIDCount() < 150 + self.diffi then
		local index = 1
		if MovableMan:ValidMO(self.Teleports[index]) and self.Teleports[index].ClassName ~= "Entity" then
			local spawn = true
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("SpawnArea" .. index, Act.Pos) then
					spawn = false
					break
				end
			end
			
			if spawn then
				Trigger.nextTimerLimit = 4000 - self.diffi*20 + self.lives*600
				
				local Act = nil
				if PosRand() < 0.8 then
					Act = self:CreateEnemyLongRange(Actor.AIMODE_GOTO)
				else
					Act = CreateACrab("AA Crawler", "Air Support.rte")
					if Act then
						Act.Team = Activity.TEAM_2
						Act.AIMode = Actor.AIMODE_GOTO
					end
				end
				
				if Act then
					Act:AddAISceneWaypoint(SceneMan.Scene:GetArea("Wpt1"):GetCenterPoint())
					Act:AddAISceneWaypoint(Vector(490, 1050))	-- Player brain
					Act:FlashWhite(200)
					Act.Pos = self.SpawnAreas[index]:GetCenterPoint()
					MovableMan:AddActor(Act)
					
					local Fx = CreateAEmitter("Teleporter Effect B")
					Fx.Pos = Act.Pos
					MovableMan:AddParticle(Fx)
				end
			else
				Trigger.nextTimerLimit = 2000
			end
		else
			Trigger.delete = true		-- The teleport was destroyed
		end
	end
end

function SpawnDefence(self, Trigger)
	if MovableMan:GetMOIDCount() < 140 + self.diffi then
		local index = 1
		if MovableMan:ValidMO(self.Teleports[index]) and self.Teleports[index].ClassName ~= "Entity" then
			local spawn
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_2 and Act.Pos.X > 2600 then
					for i = 3, 6 do
						if not SceneMan.Scene:WithinArea("Sentry" .. i, Act.Pos) and not SceneMan.Scene:WithinArea("SpawnArea" .. index, Act.Pos) then
							spawn = i
							break
						end
					end
				end
			end
			
			if spawn then
				Trigger.nextTimerLimit = math.max(SelectRand(10000, 20000) - self.diffi*100 + self.lives*2000, 6000)
				
				local Act
				if spawn == 3 then
					Act = self:CreateFlyingSentry(Actor.AIMODE_GOTO)
				else
					Act = self:CreateEnemyLongRange(Actor.AIMODE_GOTO)
				end
				
				if Act then
					local Wpt = SceneMan.Scene:GetArea("Sentry" .. spawn)
					if Wpt then
						Wpt = Wpt:GetCenterPoint()
						Act:AddAISceneWaypoint(SceneMan:MovePointToGround(Vector(Wpt.X, Wpt.Y-10), 21, 3))
						Act:FlashWhite(200)
						Act.Pos = self.SpawnAreas[index]:GetCenterPoint()
						MovableMan:AddActor(Act)
						
						local Fx = CreateAEmitter("Teleporter Effect B")
						Fx.Pos = Act.Pos
						MovableMan:AddParticle(Fx)
					end
				end
			else
				Trigger.nextTimerLimit = 3000
			end
		else
			Trigger.delete = true		-- The teleport was destroyed
		end
	end
end

function SpawnAA(self, Trigger)	-- Spawned at the right edge of the map
	if MovableMan:GetMOIDCount() < 150 + self.diffi then
		local spawn = true
		for Act in MovableMan.Actors do
			if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("Area1", Act.Pos) then
				spawn = false
				break
			end
		end
		
		if spawn then
			Trigger.nextTimerLimit = math.max(SelectRand(8000, 12000) - self.diffi*200 + gLandingPad.phase*2000 + self.lives*1000, 4000)
			
			local Act
			if PosRand() < 0.6 then
				Act = self:CreateEnemyLongRange(Actor.AIMODE_GOTO)
			else
				Act = CreateACrab("AA Crawler", "Air Support.rte")
				if Act then
					Act.Team = Activity.TEAM_2
					Act.AIMode = Actor.AIMODE_GOTO
				end
			end
			
			if Act then
				Act.Pos = self.SpawnAreas[4]:GetCenterPoint()
				Act:AddAISceneWaypoint(Vector(Act.Pos.X - 200, Act.Pos.Y))
				MovableMan:AddActor(Act)
			end
		else
			Trigger.nextTimerLimit = 6000
		end
	end
end

function SpawnSentry2(self, Trigger)
	if MovableMan:GetMOIDCount() < 160 + self.diffi then
		local index = 2
		if MovableMan:ValidMO(self.Teleports[index]) and self.Teleports[index].ClassName ~= "Entity" then
			local spawn = true
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("Sentry" .. index, Act.Pos) then
					spawn = false
					break
				end
			end
			
			if spawn then
				Trigger.nextTimerLimit = math.max(SelectRand(20000, 30000) - self.diffi*1000, 8000)
				local Act = self:CreateEnemyLongRange(Actor.AIMODE_GOTO)
				if Act then
					Act:AddAISceneWaypoint(SceneMan.Scene:GetArea("EGL" .. index):GetCenterPoint())
					Act:FlashWhite(200)
					Act.Pos = self.SpawnAreas[index]:GetCenterPoint()
					MovableMan:AddActor(Act)
					
					local Fx = CreateAEmitter("Teleporter Effect B")
					Fx.Pos = Act.Pos
					MovableMan:AddParticle(Fx)
				end
			else
				Trigger.nextTimerLimit = math.max(SelectRand(3000, 5000) - self.diffi*150, 1000)
			end
		else
			Trigger.delete = true	-- The teleport was destroyed
		end
	else
		Trigger.nextTimerLimit = 10000
	end
end

function SpawnSentry3(self, Trigger)
	if MovableMan:GetMOIDCount() < 160 + self.diffi then
		local index = 3
		if MovableMan:ValidMO(self.Teleports[index]) and self.Teleports[index].ClassName ~= "Entity" then
			local spawn = true
			for Act in MovableMan.Actors do
				if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("Sentry" .. index, Act.Pos) then
					spawn = false
					break
				end
			end
			
			if spawn then
				spawn = false
				for Act in MovableMan.Actors do
					if Act.Team == Activity.TEAM_1 and SceneMan.Scene:WithinArea("EnemyBase4", Act.Pos) then
						spawn = true
						break
					end
				end
				
				if spawn then
					Trigger.nextTimerLimit = math.max(SelectRand(10000, 30000) - self.diffi*800 + self.lives*5000, 8000)
					local Act = self:CreateEnemy(Actor.AIMODE_GOTO)
					if Act then
						Act:AddAISceneWaypoint(SceneMan.Scene:GetArea("Area3"):GetCenterPoint() + Vector(SelectRand(-50,50), -20))
						Act:FlashWhite(200)
						Act.Pos = self.SpawnAreas[index]:GetCenterPoint()
						MovableMan:AddActor(Act)
						
						local Fx = CreateAEmitter("Teleporter Effect B")
						Fx.Pos = Act.Pos
						MovableMan:AddParticle(Fx)
					end
				end
			else
				Trigger.nextTimerLimit = math.max(SelectRand(3000, 5000) - self.diffi*150, 1000)
			end
		else
			Trigger.delete = true	-- The teleport was destroyed
		end
	else
		Trigger.nextTimerLimit = 15000
	end
end

function SpawnPatrol1(self, Trigger)	-- Spawn at the first underground level
	if MovableMan:GetMOIDCount() < 150 + self.diffi then
		local spawn = true
		for Act in MovableMan.Actors do
			if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("Area4", Act.Pos) then
				spawn = false
				break
			end
		end
		
		if spawn then
			Trigger.nextTimerLimit = math.max(SelectRand(28000, 114000) - self.diffi*200 + self.lives*3000, 4000)
			
			local Act
			if PosRand() < 0.2 then
				Act = self:CreateEnemyLongRange(Actor.AIMODE_PATROL)
			else
				Act = self:CreateEnemy(Actor.AIMODE_PATROL)
			end
			
			if Act then
				Act.Pos = self.SpawnAreas[5]:GetCenterPoint()
				MovableMan:AddActor(Act)
			end
		else
			Trigger.nextTimerLimit = 20000
		end
	end
end

function SpawnPatrol2(self, Trigger)	-- Spawn at the second underground level
	if MovableMan:GetMOIDCount() < 150 + self.diffi then
		local spawn = true
		for Act in MovableMan.Actors do
			if Act.Team == Activity.TEAM_2 and SceneMan.Scene:WithinArea("Area5", Act.Pos) then
				spawn = false
				break
			end
		end
		
		if spawn then
			Trigger.nextTimerLimit = math.max(SelectRand(17000, 417000) - self.diffi*250 + self.lives*5000, 4000)
			
			local Act
			if PosRand() < 0.3 then
				Act = self:CreateEnemyLongRange(Actor.AIMODE_PATROL)
			else
				Act = self:CreateEnemy(Actor.AIMODE_PATROL)
			end
			
			if Act then
				Act.Pos = self.SpawnAreas[6]:GetCenterPoint()
				MovableMan:AddActor(Act)
			end
		else
			Trigger.nextTimerLimit = 20000
		end
	end
end

function SpawnRonin(self, Trigger)
	if gLandingPad.phase < 9 or gLandingPad.phase > 10 then
		Trigger.delete = true
	elseif MovableMan:GetMOIDCount() < 180 and self:Dead(self.Reinforce) then
		Trigger.nextTimerLimit = 20000
		local InfiltratorA = self:CreatePlayer(Actor.AIMODE_GOTO)
		local InfiltratorB = self:CreatePlayer(Actor.AIMODE_GOTO)
		if InfiltratorA and InfiltratorB then
			InfiltratorA:AddAISceneWaypoint(SceneMan.Scene:GetArea("EGL2"):GetCenterPoint()-Vector(50,0))
			InfiltratorB:AddAISceneWaypoint(SceneMan.Scene:GetArea("EGL2"):GetCenterPoint()-Vector(150,0))
			
			--self.Reinforce = CreateACRocket("Rocket MK1a", "Air Support.rte")
			self.Reinforce = CreateACDropShip("Drop Ship MK1a", "Air Support.rte")
			if self.Reinforce then
				self.Reinforce.Pos = Vector(SceneMan.Scene:GetArea("Rocket" .. SelectRand(2,4)):GetCenterPoint().X,-75)
				self.Reinforce.Vel = Vector(0,-2)
				self.Reinforce.Team = Activity.TEAM_1
				self.Reinforce:AddInventoryItem(InfiltratorA)
				self.Reinforce:AddInventoryItem(InfiltratorB)
				MovableMan:AddActor(self.Reinforce)
			end
		end
	else
		Trigger.nextTimerLimit = 3000
	end
end

function SpawnRonin2(self, Trigger)
	if gLandingPad.phase < 11 then
		Trigger.delete = true
	elseif MovableMan:GetMOIDCount() < 180 and self:Dead(self.Reinforce) then
		Trigger.nextTimerLimit = 30000
		local InfiltratorA = self:CreatePlayer(Actor.AIMODE_GOTO)
		if InfiltratorA then
			InfiltratorA:AddAISceneWaypoint(self.ObjectArea:GetCenterPoint())
			
			self.Reinforce = CreateACRocket("Rocket MK1a", "Air Support.rte")
			--self.Reinforce = CreateACDropShip("Drop Ship MK1a", "Air Support.rte")
			if self.Reinforce then
				self.Reinforce.Pos = Vector(SceneMan.Scene:GetArea("Sentry" .. SelectRand(5,7)):GetCenterPoint().X,-50)
				--self.Reinforce.Vel = Vector(0,0)
				self.Reinforce.Team = Activity.TEAM_1
				self.Reinforce.AIMode = Actor.AIMODE_DELIVER
				self.Reinforce:AddInventoryItem(InfiltratorA)
				MovableMan:AddActor(self.Reinforce)
			end
		end
	else
		Trigger.nextTimerLimit = 3000
	end
end