function Create(self)
	self.lastRot = 0
	self.timer = 2
	self.acc = 0.8
	self.step = 5	-- Calculate once every n frames
	self.airLarge = 0.95^self.step
	self.airSmall = 0.997^self.step
	
	-- Find an enemy close to the aim point
	local EnemyDist
	local ang
	local bestAng = 2
	local id = 255
	for Act in MovableMan.Actors do
		if Act.Team ~= self.Team then
			EnemyDist = SceneMan:ShortestDistance(self.Pos, Act.Pos, false)
			if EnemyDist.Magnitude < 750 then
				ang = math.abs(EnemyDist.AbsRadAngle - self.RotAngle)
				if ang < 0.8 then	-- within ~90 degrees
					ang = ang + EnemyDist.Magnitude / 750	-- heuristic
					if ang < bestAng then
						id = SceneMan:CastMORay(self.Pos, EnemyDist, self.ID, -1, false, 1)
						if id < 255 then
							Act = MovableMan:GetMOFromID(MovableMan:GetRootMOID(id)) -- lock on to any obscuring MO
							if Act.Team ~= self.Team then
								bestAng = ang
								self.Target = Act
							end
						end
					end
				end
			end
		end
	end
	
	if self.Target then
		self.Grav = SceneMan.GlobalAcc * (TimerMan.DeltaTimeSecs * self.step)
		self.factor = FrameMan.PPM * TimerMan.DeltaTimeSecs * self.step
	end
end

function Destroy(self)
	local Blast = CreateAEmitter("HEAT Blast", "Air Support.rte")
	if Blast then
		Blast.Pos = self.Pos
		Blast.RotAngle = self.lastRot
		MovableMan:AddParticle(Blast)
	end
end

function Update(self)
	self.lastRot = self.RotAngle
	local RelVel = self:UnRotateOffset(self.Vel)	-- Transform velocity to the projectiles local coordinate system (X is forward)
	if RelVel.X > 40 then
		RelVel.X = RelVel.X * 0.95	-- Air resistance for forward movement
	else
		RelVel.X = RelVel.X + self.acc	-- Acceleration
	end
	
	-- Air resistance for movement Perpendicular to RotAngle
	if math.abs(RelVel.Y) > 12 then
		RelVel.Y = RelVel.Y * 0.95
	else
		RelVel.Y = RelVel.Y * 0.997
	end
	
	self.Vel = self:RotateOffset(RelVel)	-- Transform back to the global coordinate system (X is right)
	
	if self.timer > 4 then
		self.timer = 1
		
		if MovableMan:ValidMO(self.Target) and self.Target.ClassName ~= "Entity" then
			-- Simulate missile flight
			local Vel = Vector(self.Vel.X, self.Vel.Y)
			local Pos = Vector(self.Pos.X, self.Pos.Y)
			local minDist = 10000
			local closeRotAng = 0
			local Range
			local ang
			for i = 1, 9 do
				RelVel = self:UnRotateOffset(Vel)
				RelVel.X = RelVel.X + self.acc * self.step	-- Acceleration
				
				-- Air resistance
				if RelVel.X > 40 then
					RelVel.X = RelVel.X * self.airLarge
				end
				
				if math.abs(RelVel.Y) > 12 then
					RelVel.Y = RelVel.Y * self.airLarge
				else
					RelVel.Y = RelVel.Y * self.airSmall
				end
				
				Vel = self:RotateOffset(RelVel)
				Pos = Pos + self.Grav + Vel * self.factor
				
				Range = SceneMan:ShortestDistance(Pos, self.Target.Pos, false)
				local dist = Range.Magnitude
				if dist < minDist then
					minDist = math.floor(dist)
					ang = Range.AbsRadAngle
					closeRotAng = Vel.AbsRadAngle
				else
					break
				end
				
				--[[ Debug --
				if not self.AimDots then
					self.AimDots = {}
				end
				
				local name = "Gray"
				if not MovableMan:ValidMO(self.AimDots[i]) or self.AimDots[i].PresetName ~= name then
					self.AimDots[i] = CreateMOSParticle(name, "Debug.rte")
					MovableMan:AddParticle(self.AimDots[i])
				end
				
				self.AimDots[i].Age = 1
				self.AimDots[i].Pos = Pos]]
			end
			
			if closeRotAng > ang + 3.1416 then
				ang = ang - (closeRotAng-6.2832)
			elseif ang > closeRotAng + 3.1416 then
				ang = ang - 6.2832 - closeRotAng
			else
				ang = ang - closeRotAng
			end
			
			if ang < -0.05 then
				self.RotAngle = self.RotAngle + math.max(ang/2, -0.2)
			elseif ang > 0.05 then
				self.RotAngle = self.RotAngle + math.min(ang/2, 0.2)
			end
		else
			-- OrientToVel
			local minAng = math.min(self.Vel.AbsRadAngle, self.RotAngle)
			self.RotAngle = minAng + (math.max(self.Vel.AbsRadAngle, self.RotAngle) - minAng)/2
			self.Target = nil
		end
	else
		self.timer = self.timer + 1
	end
end