function Create(self)
	self:EnableEmission(false)
	self.LastVel = Vector(self.Vel.X, self.Vel.Y)
	self.LifeTimer = Timer()
	self.range = (FrameMan.ResX*3 + FrameMan.ResY*2)/5	-- Target search range 
	self.last = self.RotAngle	-- Initialize the PID controller
	self.integral = 0	-- Initialize the PID controller
	
	-- Convert any angle to [-Pi .. Pi]
	self.AngNorm = function(ang)
		while ang < -3.1416 do ang = ang + 6.2832 end
		while ang > 3.1416 do ang = ang - 6.2832 end
		return ang
	end
	
	-- Calculate the smallest difference between two angles
	self.AngDiff = function(ang1, ang2, abs)
		local diff = self.AngNorm(ang2) - self.AngNorm(ang1)
		diff = self.AngNorm(diff)
		if abs and diff < 0 then
			return -diff
		end
		
		return diff
	end
end

function Update(self)
	if self.LifeTimer:IsPastSimMS(3000) then
		self:GibThis()
		return
	end
	
	self.update = not self.update
	if self.update then
		self.AngularVel = self.AngularVel * 0.98
		
		if MovableMan:IsActor(self.Target) then
			local FutureVel = self.Vel  + (self.Vel-self.LastVel)*10	-- Estimate future velocity
			local targAng = SceneMan:ShortestDistance(self.Pos+FutureVel*3,
												self.Target.Pos+self.Target.Vel*3, false).AbsRadAngle
			local diff = self.AngDiff(targAng, FutureVel.AbsRadAngle+self.AngularVel/15)
			self.EmitAngle = math.max(math.min(3.142 - diff*2, 4.142), 2.142)	-- Vector thrust
			
			-- The controler tries to match the angle of the velocity vector with the angle to the target
			self.integral = self.integral + diff
			local change = math.max(math.min(0.2*diff+0.01*self.integral+0.1*(diff-self.last), 0.4), -0.4)
			self.last = diff
			
			-- Rotate towards target angle
			if change > 0 then
				self.AngularVel = math.max(self.AngularVel - change*12, -8.5)
			elseif change < 0 then
				self.AngularVel = math.min(self.AngularVel - change*12, 8.5)
			end
			
			-- Set throtle based on angle to target vector
			diff = self.AngDiff(targAng, self.RotAngle, true)
			if diff > 1.571 then
				self.EmitAngle = 3.142	-- Go straight
				self:EnableEmission(false)
			else
				self.GetThrottle = math.max(math.min(-4*diff+0.7, 0.9), -0.9)
				self:EnableEmission(true)
			end
		else
			self:EnableEmission(false)
			local minDist = self.range
			for Act in MovableMan.Actors do	-- Look for a new target in front of the missile
				if Act.Team ~= self.Team then
					local Range = SceneMan:ShortestDistance(self.Pos, Act.Pos, false)
					local dist = Range.Magnitude
					if dist < minDist and 
						self.AngDiff(self.RotAngle, Range.AbsRadAngle, true) < 1
					then	-- The missile looks for targets within a ~120deg cone
						minDist = dist
						self.Target = Act
					end
				end
			end
		end
	elseif MovableMan:IsActor(self.Target) then
		local targRng = SceneMan:ShortestDistance(self.Pos, self.Target.Pos, false).Magnitude
		if self.closest then
			if targRng < self.closest then
				self.closest = targRng	-- Still moving towards the target
			else
				self:GibThis()	-- Moving away from the target; detonate
			end
		elseif targRng < self.Target.Radius + 40 then
			self.closest = targRng	-- Proximity fuse armed
		end
	end
	
	self.LastVel:SetXY(self.Vel.X, self.Vel.Y)
end