UAV = {}

function UAV.Initialize(self)
	self.MGTimer = Timer()
	self.DropTimer = Timer()
	
	self.minThrot = 3900
	self.maxThrot = 6700
	self.throttle = self.minThrot
	self.ammo = 300		-- Machine Guns
	self.liftMultiplier = 15.3
	self.liftMaximum = 15000
	self.stallSpeed = 6
	self.angle = 0
	self.speed = 0
	self.payload = 0
	self.bomb = ""
	self.playerCtrl = false
	self.Ctrl = self:GetController()
	self.damage = 0
	self.dmgThres = 80
	self.TailPos = Vector(-40,0)
	self.AimDistance = Vector(FrameMan.PlayerScreenWidth/2.5, FrameMan.PlayerScreenHeight/2)
	self.Vel = self:RotateOffset(Vector(self.stallSpeed + 1, 0))
	
	if self.HFlipped then 
		self.RotAngle = 0.1
	else
		self.RotAngle = -0.1
	end
	
	-- Secondary key
	self.secondary = UAV.SecondaryKey(self)
	
	-- Minimap
	self.GroundDots = {}
	self.DotPos = {}
	self.dotRes = 4		-- n pixel spacing in the minimap
	self.xDots = math.ceil(SceneMan.SceneWidth / 350)		-- one dot every n pixels in the map
	self.yDots = math.ceil(SceneMan.SceneHeight / 350)
	
	local xgap = math.floor(SceneMan.SceneWidth / self.xDots)
	local Radar = CreateMOPixel("Ground", "Air Support.rte")
	Radar.Pos.Y = 0
	
	self.xDots = self.xDots * self.dotRes
	self.yDots = self.yDots * self.dotRes
	self.xDiv = SceneMan.SceneWidth / self.xDots
	self.yDiv = SceneMan.SceneHeight / self.yDots
	
	for i = 1, self.xDots/self.dotRes do
		Radar.Pos.X = math.floor(xgap*(i-1) + xgap/2)
		table.insert(self.DotPos, i, Vector(math.ceil(Radar.Pos.X/self.xDiv), math.ceil(Radar:GetAltitude(0,7)/self.yDiv+self.yDots+20)))
	end
end

function UAV.Destroy(self)
	self.playerCtrl = false
	for i = 1, #self.GroundDots do
		self.GroundDots[i].ToDelete = true
		self.GroundDots[i] = nil
	end
end

function UAV.SecondaryKey(self)
	if self.Ctrl:IsMouseControlled() then
		return Controller.BODY_CROUCH
	end
	
	return Controller.AIM_SHARP
end

function UAV.Offset(Origin, ang, dist)
	local Tmp = Vector(dist, 0):RadRotate(ang)	
	if Origin then
		return Vector(Tmp.X + Origin.X, Tmp.Y + Origin.Y)
	end
	
	return Tmp
end

function UAV.AddIcon(self, Icon, dist)
	Icon.Vel = self.Vel
	Icon.Pos = self.Pos + self:RotateOffset(Vector(0, -math.abs(Icon.SpriteOffset.Y) - math.abs(self.SpriteOffset.Y) - dist))
	MovableMan:AddParticle(Icon)
end

function UAV.Move(self)
    if self.Health < 1 and self.Vel.Magnitude < 25 then
        self:GibThis()
    end
	
	if self.RotAngle > 3.1415 or self.RotAngle < -3.1415 then
		self.RotAngle = 0
		self.HFlipped = not self.HFlipped
	end
	
	self.angle = self.RotAngle
	if self.HFlipped then
		self.angle = self.angle + 3.1415
	end
	
	-- Movement in direction of view angle
	self.speed = math.abs(self.Vel.Magnitude * math.cos(math.abs(self.Vel.AbsRadAngle - self.RotAngle)))
	
	if self.speed > self.stallSpeed then
		self.AngularVel = self.AngularVel * 0.95
	end
	
	-- Direct the velocity towards the view angle
	if self.speed > self.stallSpeed then
		local mag = self.Vel.Magnitude
		local Momentum = Vector(mag * 0.03, 0):RadRotate(self.angle) + self.Vel
		self.Vel = Momentum:SetMagnitude(mag)
	else	-- Stall
		if self.angle < 1.5708 then
			if (self.angle > -1.0472 or self.angle > 5.2360) then
				self.AngularVel = self.AngularVel - 0.024
			end
		elseif self.angle < 3.6652 then
			self.AngularVel = self.AngularVel + 0.024
		end
	end
	
	-- Angle of attack
	self.attack = math.abs(self.Vel.AbsRadAngle - self.angle)
	if self.attack > 6.2832 then
		self.attack = self.attack - 6.2832
	elseif self.attack < -1.5708 then
		self.attack = 6.2832 - self.attack
	end
	
	-- Add engine acceleration
	local Force
	if self.speed < 20 then
		Force = Vector(self.throttle, 0):RadRotate(self.angle)
	else
		Force = Vector(self.throttle*0.77, 0):RadRotate(self.angle)
		self.Vel = self.Vel - self:RotateOffset(Vector(self.speed*0.0044, 0))	-- Extra air resistance at high speeds
	end
	
	-- Add lift
	if self.speed > self.stallSpeed * 0.8 and self.attack < 1 then
		local lift = math.min(self.speed^2 * math.max(1-self.attack, 0) * self.liftMultiplier, self.liftMaximum)
		Force = Force + self:RotateOffset(Vector(0, -lift))
	end
	
	-- Add extra drag when angle of attack is high
	if self.attack > 0.2 and self.Vel.Magnitude > 8 then
		if self.attack > 1.5708 then
			self.attack = 3.1415 - self.attack
		end
		
		Force = Force + Vector((self.Vel.Magnitude - self.speed)^1.6 * (self.attack+0.1) * 500 + 1500, 0):RadRotate(self.Vel.AbsRadAngle + 3.1415)
	end
	
	self:AddAbsForce(Force, self.Pos)
	
	-- Damage effects
	if self.Health < self.dmgThres then
		if self.damage > 0 then
			local NewPos = self.Pos + self:RotateOffset(self.TailPos)
			local Trail = CreateMOSParticle("Tiny Smoke Ball Fast", "Air Support.rte")
			Trail.Pos = NewPos + NewPos.Perpendicular:Normalize()*RangeRand(-4, 4)
			Trail.Vel = self.Vel
			MovableMan:AddParticle(Trail)
			self.damage = self.damage - 1
		elseif self.Health/90 < PosRand() then
			self.dmgThres = self.Health
			self.damage = (100 - self.Health + RangeRand(10, 30)) * 10
		end
	end
end

function UAV.PlayerControlled(self)
	self.playerCtrl = true
	
	-- Fake sharp aim
	self.ViewPoint = self.Pos + Vector(0,self.AimDistance.Y/3) + Vector(math.cos(self.angle) * self.AimDistance.X, -math.sin(self.angle) * self.AimDistance.Y)
	
	-- Throttle
	if self.Ctrl:IsState(Controller.MOVE_UP) then
		self.throttle = self.maxThrot
		
		-- Trail
		local NewPos = self.Pos + self:RotateOffset(self.TailPos)
		local Trail = CreateMOPixel("Small Trail", "Air Support.rte")
		Trail.Pos = NewPos + NewPos.Perpendicular:Normalize()*RangeRand(-3, 4)
		Trail.Vel = self.Vel
		MovableMan:AddParticle(Trail)
	else
		self.throttle = self.minThrot
	end
	
	-- Turn slower at high and low speeds
	if self.speed > self.stallSpeed then
		if self.AngularVel < 1.5 and self.Ctrl:IsState(Controller.MOVE_LEFT) then
			self.AngularVel = self.AngularVel + math.max(0.26 - math.abs(20 - self.speed) / 45, 0.07)
		elseif self.AngularVel > -1.5 and self.Ctrl:IsState(Controller.MOVE_RIGHT) then
			self.AngularVel = self.AngularVel - math.max(0.26 - math.abs(20 - self.speed) / 45, 0.07)
		end
	end
end

function UAV.FireMG(self)
	if self.MGTimer:IsPastSimMS(83) then	-- ~2x720 rpm
		self.MGTimer:Reset()
		
		local Emitter = nil
		if self.ammo % 3 == 0 then
			Emitter = CreateAEmitter("UAV MG Tracer", "Air Support.rte")
		else
			Emitter = CreateAEmitter("UAV MG", "Air Support.rte")
		end
		
		Emitter.Vel = self.Vel
		Emitter.Pos = self.Pos + self:RotateOffset(Vector(math.abs(self.SpriteOffset.X) + 16, 12))
		Emitter.RotAngle = self.angle
		MovableMan:AddParticle(Emitter)
		
		local Casing = CreateMOSParticle("MG Casing", "Air Support.rte")
		Casing.Pos = self.Pos + self:RotateOffset(Vector(13, math.abs(self.SpriteOffset.Y) + 1))
		Casing.AngularVel = RangeRand(2, 12)
		Casing.Vel = self.Vel * RangeRand(0.77, 0.97)
		MovableMan:AddParticle(Casing)
		
		Casing = CreateMOSParticle("MG Casing", "Air Support.rte")
		Casing.Pos = self.Pos + self:RotateOffset(Vector(13, math.abs(self.SpriteOffset.Y) + 1))
		Casing.AngularVel = RangeRand(2, 12)
		Casing.Vel = self.Vel * RangeRand(0.77, 0.97)
		MovableMan:AddParticle(Casing)
		
		self.ammo = self.ammo - 1
	end
end

function UAV.DropBomb(self)
	local Bomb = CreateMOSRotating(self.bomb, "Air Support.rte")
	Bomb.Pos = self.Pos + self:RotateOffset(Vector(5, math.abs(self.SpriteOffset.Y) - Bomb.SpriteOffset.Y))
	Bomb.RotAngle = self.angle
	Bomb.Vel = self.Vel
	MovableMan:AddParticle(Bomb)
	
	self.payload = self.payload - 1
end

function UAV.Map(self)
	if SceneMan.SceneWrapsX then
		local startDot = math.ceil((self.xDots/self.dotRes)*(self.Pos.X/SceneMan.SceneWidth))
		if self.Vel.X > 0 then	-- moving to the right
			for i = startDot, #self.DotPos do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "Air Support.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = self.Pos.X + (i-startDot)*self.dotRes
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				self.GroundDots[i].Age = 1
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
			
			local lastDotX = self.GroundDots[#self.DotPos].Pos.X
			for i = 1, startDot - 1 do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "Air Support.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = lastDotX + i*self.dotRes
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				self.GroundDots[i].Age = 1
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
		else	-- moving to the left
			local pos = 0
			for i = startDot, 1, -1 do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "Air Support.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = self.Pos.X - pos
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				self.GroundDots[i].Age = 1
				pos = pos + self.dotRes
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
			
			for i = #self.DotPos, startDot + 1, -1 do
				if not MovableMan:ValidMO(self.GroundDots[i]) then
					self.GroundDots[i] = CreateMOPixel("Ground", "Air Support.rte")
					MovableMan:AddParticle(self.GroundDots[i])
				end
				
				self.GroundDots[i].Vel = self.Vel
				self.GroundDots[i].Pos.X = self.Pos.X - pos
				self.GroundDots[i].Pos.Y = self.Pos.Y + self.DotPos[i].Y
				self.GroundDots[i].Age = 1
				pos = pos + self.dotRes
				
				if self.GroundDots[i].Pos.X > SceneMan.SceneWidth then
					self.GroundDots[i].Pos.X = self.GroundDots[i].Pos.X - SceneMan.SceneWidth
				elseif self.GroundDots[i].Pos.X < 0 then
					self.GroundDots[i].Pos.X = SceneMan.SceneWidth + self.GroundDots[i].Pos.X
				end
			end
		end
		
		-- Update UAV dot
		local Px = CreateMOPixel("PlayerPos", "Air Support.rte")
		Px.Pos = Vector(self.Pos.X, self.Pos.Y + self.Pos.Y/self.yDiv + self.yDots + 20)
		Px.Vel = self.Vel
		MovableMan:AddParticle(Px)
	else
		for i = 1, #self.DotPos do
			if self.GroundDots[i] == nil or not MovableMan:ValidMO(self.GroundDots[i]) then
				self.GroundDots[i] = CreateMOPixel("Ground", "Air Support.rte")
				MovableMan:AddParticle(self.GroundDots[i])
			end
			
			self.GroundDots[i].Pos = self.DotPos[i] + Vector(self.Pos.X - self.xDots/2, self.Pos.Y)
			self.GroundDots[i].Vel = self.Vel
			self.GroundDots[i].Age = 1
		end
		
		-- Update UAV dot
		local Px = CreateMOPixel("PlayerPos", "Air Support.rte")
		Px.Pos = self.Pos + Vector(self.Pos.X/self.xDiv - self.xDots/2, self.Pos.Y/self.yDiv + self.yDots + 20)
		Px.Vel = self.Vel
		MovableMan:AddParticle(Px)
	end
end
